@@ -4,7 +4,7 @@
 
 package Schedule::Depend;
 
-our $VERSION = 0.02;
+our $VERSION = 0.04;
 
 # make messages output during the use phase prettier.
 
@@ -15,6 +15,8 @@ use Carp;
 
 use File::Basename;
 
+# pretty-print the object in debug mode.
+
 use Data::Dumper;
 	$Data::Dumper::Purity		= 1;
 	$Data::Dumper::Terse		= 1;
@@ -226,7 +228,10 @@ sub precheck
 	my $que = shift;
 	my $job = shift;
 
-	my $verbose = $que->{verbose};
+	# this stuff only goes out of the verbosity is
+	# at the "detail" level.
+
+	my $verbose = $que->{verbose} > 1;
 
 	my $pidfile =
 		$que->{filz}{$job} = $que->{alias}{rundir} . "/$job.pid";
@@ -272,7 +277,7 @@ sub precheck
 			# otherwise we can just zero out the pidfile and 
 			# keep going.
 
-			print "$$: Completed: $job" if $que->{verbose};
+			print "$$: Completed: $job" if $verbose;
 
 			if( $que->{restart} )
 			{
@@ -284,6 +289,7 @@ sub precheck
 				if( $exit eq '0' )
 				{
 					# no reason to re-run this job.
+					# note: this is always reprinted.
 					
 					print "$$: Marking job for skip on restart: $job";
 
@@ -291,21 +297,26 @@ sub precheck
 				}
 				else
 				{
-					print "$$:	$job previous non-zero exit, will be re-run";
+					print "$$:	$job previous non-zero exit, will be re-run"
+						if $verbose;
 				}
 			}
 			else
 			{
-				print "$$: Not Running:  $job" if $que->{verbose};
+				print "$$: Not Running:  $job" if $que->{verbose}
+					if $verbose;
 			}
 		}
 		elsif( @linz )
 		{
+			# preparing the queue when one of the scheduled
+			# jobs is running "error".
+			#
 			# here's where the fun part starts. for now i'll
 			# punt: anything without an exit status is assumed
 			# to be running.
 
-			print STDERR "$$:	Pidfile without exit: $job";
+			print STDERR "\n$$:	Pidfile without exit: $job";
 
 			$running = 1;
 		}
@@ -337,7 +348,8 @@ sub precheck
 
 	if( $que->{skip}{$job} || $running )
 	{
-		print STDERR "\n$$: Leaving existing pidfile untouched\n";
+		print "\n$$: Leaving existing pidfile untouched\n"
+			if $verbose;
 	}
 	else
 	{
@@ -451,12 +463,23 @@ sub prepare
 	# pidfiles show a zero exit. this allows any dependencies
 	# to be maintained w/o having to re-run the entire que
 	# if it aborts.
+	#
+	# $argz{verbose} overrides all other levels during
+	# preparation, without an argument it's either 
+	# 2 (set via $que->{debug}) or defaults to 0 (not much
+	# output).
+	#
+	# verbose > 0 will display the input lines.
+	# verobse > 1 additionally displays each alias/dependency
+	# as it is processed.
 
 	$que->{debug}   =	$alias->{debug} ? 1 :
 						$argz{debug}	? 1 :
 						$^P;
 
-	$que->{verbose} =	$argz{verbose}	? 1 : $que->{debug};
+	$que->{verbose} =	$argz{verbose} ? $argz{verbose} :
+						$que->{debug}  ? 2 :
+						0;
 
 	$que->{restart} = $argz{restart} ? 1 : 0;
 
@@ -478,7 +501,7 @@ sub prepare
 		or croak "build_queue called with empty depend list.";
 
 	print "\nPreparing Schedule From:", @linz, ''
-		if $que->{verbose};
+		if $que->{verbose} > 0;
 
 	# step 1: deal with information in the aliases.
 	#
@@ -507,14 +530,15 @@ sub prepare
 		map { ( split /\s*=\s*/, $_, 2 ) } grep /=/, @linz
 	);
 
-	print STDERR "\n$$: Aliases:", Dumper $alias;
+	print "\n$$: Aliases:", Dumper $alias
+		if $que->{verbose} > 1;
 
 	croak "$$: Negative maxjob: $alias->{maxjob}" 
 		if $alias->{maxjob} < 0;
 
 	for( $alias->{rundir}, $alias->{logdir} )
 	{
-		print "$$: Checking: $_" if $que->{verbose};
+		print "$$: Checking: $_" if $que->{verbose} > 1;
 
 		-e		|| croak "Non-existant:  $_";
 		-w _	|| croak "Un-writable:   $_";
@@ -528,7 +552,8 @@ sub prepare
 	# items without '=' are the sequence information, items
 	# with '=' in them have already been dealt with above.
 
-	print "\n$$: Starting rule processing" if $que->{verbose};
+	print "\n$$: Starting rule processing"
+		if $que->{verbose} > 1;
 
 	for( grep { ! /=/ } @linz )
 	{
@@ -537,7 +562,8 @@ sub prepare
 		croak "$$: Bogus rule '$_' has no targets"
 			unless $a;
 
-		print "\n$$: Processing rule: '$_'\n" if $que->{verbose};
+		print "\n$$: Processing rule: '$_'\n"
+			if $que->{verbose} > 1;
 
 		# step 1: validate the job status. this includes
 		# checking if any are already running or if we
@@ -678,23 +704,17 @@ sub debug
 {
 	my $que = shift;
 
-	print "\n$$: Debugging Schedule.\n";
-
 	my $tmp = eval ( $tmp = Dumper $que )
 		or croak "Failed to generate tmp que for debug: $!";
 
 	$tmp->{debug}	= 1;
-	$tmp->{verbose}	= 1;
+	$tmp->{verbose}	= 99;	# guaranteed to be the max :-)
 	$tmp->{maxjob}	= 1;
 
-	$tmp->execute;
-
-	print "\n$$: Debugging Complete.\n";
-	print "\n$$: Debug found error: $@\n" if $@;
+	eval { $tmp->execute };
 
 	# caller gets back original object for daisy-chaining or
 	# undef (which will abort further execution).
-	# return value).
 
 	$@ ? undef : $que;
 }
@@ -710,8 +730,35 @@ sub execute
 
 	my $que = shift;
 
-	print STDERR "\n$$: Executing:\n", Dumper $que, "\n"
-		if $que->{debug};
+	# set the verbosity level. rather than test 
+	# for $verbose > X a zillion times this sets
+	# a few, hopefully more descriptive, var's.
+	#
+	# if nothing is set then skips due to abort, 
+	# non-zero exits, forkaphobia, exec fails
+	# and croaks are displayed.
+	#
+	# print_progress	=>	show jobs being forked, reaped, 
+	#						jobs skipped on restart.
+	# print_detail 		=>	show unalias results, exit status, 
+	#						runnable jobs, job slot limitations.
+	#
+	# debug mode always runs w/ verbose == 2 (i.e., sets
+	# $print_detail).
+
+	my $print_progress	= $que->{verbose} > 0;
+	my $print_detail	= $que->{verbose} > 1;
+
+	# this intentionally goes to STDERR so that the
+	# logs have a start/completion message in them
+	# at least.
+
+	print STDERR $que->{debug}  ?
+		"\n$$: Debugging:\n", Dumper $que, "\n" :
+		"\n$$: Beginning Execution\n";
+
+	# housekeeping: set run-specific variables to 
+	# reasonable values.
 
 	local $que->{abort} = 0;
 
@@ -739,10 +786,6 @@ sub execute
 	my $maxjob = $que->{alias}{maxjob};
 	my $curjob = 0;
 
-	# syntatic sugar, minor speedup.
-
-	my $verbose = $que->{verbose};
-
 	# things are runnable when the hash stored for them in the queue
 	# has no dependencies remaining.
 	#
@@ -772,7 +815,7 @@ sub execute
 			# @runnable.
 
 			print join "\t", "\n$$: Runnable:", @runnable, "\n"
-				if $verbose;
+				if $print_detail;
 
 			if( $maxjob && (my $slots = $maxjob - $curjob) > 0 )
 			{
@@ -784,7 +827,7 @@ sub execute
 				{
 					splice @runnable, $slots, @runnable;
 
-					if( $verbose )
+					if( $print_detail )
 					{
 						print "\n$$: Startup slots:    $slots\n";
 						print "\n$$: Queue limited to: ", join "\t", @runnable, "\n";
@@ -796,7 +839,8 @@ sub execute
 				# we can't run anything if there aren't slots
 				# avilable. wait for something to exit.
 
-				print "\n$$: No slots available: Unable to start runnable jobs\n";
+				print "\n$$: No slots available: Unable to start runnable jobs\n"
+					if $print_detail;
 
 				@runnable = ();
 			}
@@ -812,7 +856,8 @@ sub execute
 
 				my $run = $que->unalias( $job );
 
-				print STDERR "\n$$: Unalias: $job => $run\n" if $verbose;
+				print "\n$$: Unalias: $job => $run\n"
+					if $print_detail;
 
 				# open the pidfile first, better to croak here than
 				# leave a job running w/o a pidfile.
@@ -832,7 +877,8 @@ sub execute
 					# the pidfile also doesn't get updated since it
 					# already contains enough information.
 
-					print STDERR "\n\t\tSkipping $job ($run) on restart.";
+					print "\n\t\tSkipping $job ($run) on restart."
+						if $print_progress;
 
 					$que->dequeue( $job );
 					$que->complete( $job );
@@ -862,7 +908,7 @@ sub execute
 					# we do have to update the pidfile, however, to show
 					# that the job was aborted.
 
-					print STDERR "\n\t\tSkipping $job ($run) due to schedule abort.";
+					print "\n\t\tSkipping $job ($run) due to schedule abort.";
 
 					print $fh "$$\nabort $job ($run)\n-1\n";
 
@@ -900,10 +946,10 @@ sub execute
 
 						++$curjob;
 
-						if( $verbose )
+						if( $print_detail )
 						{
-							print STDERR "\n$$: Forked $pid: $job\n";
-							print STDERR "\n$$: Queued Jobs: $curjob\n";
+							print "\n$$: Forked $pid: $job\n";
+							print "\n$$: Queued Jobs: $curjob\n";
 						}
 					}
 					elsif( defined $pid )
@@ -918,7 +964,7 @@ sub execute
 						# an exit since it stops running this code.
 
 						print "\n$$: Executing: $job ($run)\n"
-							if $verbose;
+							if $print_detail;
 
 						# single-argument form allows the shell
 						# to expand any meta-char's in $run.
@@ -997,8 +1043,8 @@ sub execute
 			my $job = $jobz->{$pid}
 				or die "$$: unknown pid $pid";
 
-			print STDERR "\n$$: Exit: $job ($pid) $status\t$que->{filz}{$job}\n"
-				if( $verbose );
+			print "\n$$: Exit: $job ($pid) $status\t$que->{filz}{$job}\n"
+				if $print_detail;
 
 			open my $fh, '>>',  $que->{filz}{$job}
 				or croak "$que->{filz}{$job}: $!";
@@ -1023,8 +1069,8 @@ sub execute
 			}
 			else
 			{
-				print STDERR "\n$$: Successful: $job.\n"
-					if $verbose;
+				print "\n$$: Successful: $job ($pid).\n"
+					if $print_progress;
 			}
 
 			$que->complete( $job );
@@ -1033,7 +1079,7 @@ sub execute
 
 			--$curjob;
 
-			if( $verbose )
+			if( $print_detail )
 			{
 				print "\n$$: Current Jobs: $curjob\n";
 				print "\n$$: Pending Jobs: ",
@@ -1043,8 +1089,9 @@ sub execute
 		}
 	}
 
-	print "\n\n$$: The queue has been debugged successfully"
-		if( $que->{debug} );
+	print STDERR $que->{debug} ?
+		"\n\n$$: Debugging Completed.\n" :
+		"\n\n$$: Execution Completed.\n";
 
 	0
 }
@@ -1130,6 +1177,23 @@ referent) or with the "depend" key as a hash value:
 
 Turns on verbose execution for preparation and execution.
 
+All output controlled by verbosity is output to STDOUT;
+errors, roadkill, etc, are written to STDERR.
+
+verbose == 0 only displays a few fixed preparation and 
+execution messages. This is mainly intended for production
+system with large numbers of jobs where searching a large
+output would be troublesome.
+
+verbose == 1 displays the input schedule contents during
+preparation and fork/reap messages as jobs are started.
+
+verbose == 2 is intended for monitoring automatically
+generated queues and debugging new schedules. It displays
+the input lines as they are processed, forks/reaps, 
+exit status and results of unalias calls before the jobs
+are exec-ed.
+
 =head1 Description
 
 Parallel scheduler with simplified make syntax for job 
